package model;

public class Board {

	public static final int PAWN = 0, ROOK = 1, KNIGHT = 2, BISHOP = 3, QUEEN = 4, KING = 5;
	public static final int A = 1, B = 2, C = 3, D = 4, E = 5, F = 6, G = 7, H = 8;

	private final Piece[] pieces;
	private Position selected;

	public Board() {
		selected = null;
		// initialisation of the list of pieces
		pieces = new Piece[64];
		for(int i = 0 ; i < 64 ; i++) {
			pieces[i] = null;
		}
		int y;
		for(int color = Piece.WHITE ; color == Piece.WHITE || color == Piece.BLACK ; color++) {
			// placement of pawns
			if(color == Piece.WHITE) { y = 2; }
			else { y = 7; }
			for(int x = A ; x <= H ; x++) {
				createPiece(x, y, color, PAWN); // fills line 2 and line 7 with pawns
			}
			// placement of rook, knight, bishop, queen, king - line 1 and line 8
			if(color == Piece.WHITE) { y = 1; }
			else { y = 8; }
			createPiece(A, y, color, ROOK);
			createPiece(B, y, color, KNIGHT);
			createPiece(C, y, color, BISHOP);
			createPiece(D, y, color, QUEEN);
			createPiece(E, y, color, KING);
			createPiece(F, y, color, BISHOP);
			createPiece(G, y, color, KNIGHT);
			createPiece(H, y, color, ROOK);
		}
	}

	public void setPiece(Piece piece) {
		pieces[piece.getPosition().getID()] = piece;
	}

	public Piece getPiece(Position position) {
		return pieces[position.getID()];
	}

	public Piece getPiece(int id) {
		return pieces[id];
	}
	public void createPiece(int x, int y, int color, int type) {
		Position p = new Position(x, y);
		switch(type) {
			case ROOK -> setPiece(new Rook(p, color));
			case KNIGHT -> setPiece(new Knight(p, color));
			case BISHOP -> setPiece(new Bishop(p, color));
			case QUEEN -> setPiece(new Queen(p, color));
			case KING -> setPiece(new King(p, color));
			default -> setPiece(new Pawn(p, color));
		}
	}

	public Position getSelected() {
		return selected;
	}

	public void setSelected(Position p) {
		selected = p;
	}

	public void movePiece(Position origin, Position arrival) {
		Piece piece = getPiece(origin);
		piece.setPosition(arrival); // changes position to arrival
		setPiece(piece); // copies the piece (from origin ID) to arrival ID on the pieces list
		pieces[origin.getID()] = null; // removes the origin ID piece on the pieces list
	}

	public boolean isFree(Position p) {
		return (p.isCorrect()
				&& getPiece(p) == null);
	}

	public boolean isEnemy(Position arrival, Position origin) {
		return (arrival.isCorrect()
				&& !isFree(arrival)
				&& getPiece(arrival).getColor() != getPiece(origin).getColor() );
	}
	public boolean isKing(Position p) {
		return (p.isCorrect()
				&& !isFree(p)
				&& getPiece(p).getClass() == King.class);
	}

	public boolean isRook(Position p) {
		return (p.isCorrect()
				&& !isFree(p)
				&& getPiece(p).getClass() == Rook.class);
	}

	public boolean isPawn(Position p) {
		return (p.isCorrect()
				&& !isFree(p)
				&& getPiece(p).getClass() == Pawn.class);
	}
}
