package model;

import java.util.ArrayList;

public class Queen extends Piece{
    protected Queen(Position p, int color) {
        super(p, color);
    }


    public ArrayList<Position> getAvailableMoves(Board b) {

        // creates a dynamic list of available moves
        ArrayList<Position> moves = new ArrayList<>();
        Position arrival;

        // list of the 8 queen directions
        int[] all_x = {0, 1, 1, 1, 0, -1, -1, -1};
        int[] all_y = {1, 1, 0, -1, -1, -1, 0, 1};

        for(int n = 0 ; n < all_x.length ; n++) { // for each direction
            int x = all_x[n];
            int y = all_y[n];
            boolean blocked = false;
            int i = 1;
            while (i < 8 && !blocked) { // continues "as a ray" in that direction until blocked
                arrival = new Position(p.getX() + (x * i), p.getY() + (y * i));
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
                if (!b.isFree(arrival)) {
                    blocked = true;
                }
                i++;
            }
        }

        // Full wrote version : --------------------------------------------
        /*
        int i = 1;
        boolean left = false;
        boolean right = false;
        boolean top = false;
        boolean down = false;
        boolean leftTop = false;
        boolean rightTop = false;
        boolean leftDown = false;
        boolean rightDown = false;

        while (i < 8) {
            // Bishop moves
            arrival = new Position(p.getX() + i, p.getY() + i);
            if (!leftTop) {
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
            }
            if (!b.isFree(arrival)) {
                leftTop = true;
            }
            //right diagonal
            arrival = new Position(p.getX() - i, p.getY() + i);
            if (!rightTop) {
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
            }
            if (!b.isFree(arrival)) {
                rightTop = true;
            }
            arrival = new Position(p.getX() + i, p.getY() - i);
            if (!leftDown) {
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
            }
            if (!b.isFree(arrival)) {
                leftDown = true;
            }
            arrival = new Position(p.getX() - i, p.getY() - i);
            if (!rightDown) {
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
            }
            if (!b.isFree(arrival)) {
                rightDown = true;
            }
            // RooK moves
            arrival = new Position(p.getX() +i, p.getY());
            if (!left) {
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
            }
            if (!b.isFree(arrival)) {
                left = true;
            }
            //right diagonal
            arrival = new Position(p.getX()- i , p.getY() );
            if (!right) {
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
            }
            if (!b.isFree(arrival)) {
                right = true;
            }
            arrival = new Position(p.getX(), p.getY() + i);
            if (!top) {
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
            }
            if (!b.isFree(arrival)) {
                top = true;
            }
            arrival = new Position(p.getX(), p.getY() - i);
            if (!down) {
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
            }
            if (!b.isFree(arrival)) {
                down = true;
            }
            i++;
        }
        */
        // ------------------------------------------------------------

        return moves;
    }
}
